/* Copyright (C) 2005 Guillaume Pinot
   This file is part of pot.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
   02111-1307 USA.  */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "kelsal.h"

regex_t silent_line;
regex_t used_line;

struct kelsal_list * kl = NULL;

char *
kelsal(const char * ordi)
{
  struct kelsal_list * search;
  char * answer = NULL;

  if (kl == NULL)
    read_kelsal();

  for (search = kl; search != NULL && answer == NULL; search = search->next)
    if (!regexec(&(search->ordi), ordi, 0, NULL, 0))
      answer = search->salle;
  
  if (answer == NULL)
    {
      char * o = strdup(ordi);
      if (o == NULL)
	{
	  fprintf( stderr , "Allocation de memoire impossible\n" );
	  exit(-1);
	}

      if ((answer = (char *) malloc(sizeof(strtok(o,".") + sizeof(char)*2)))
	  == NULL)
        return NULL;
      strcpy(answer,"(");
      if (strlen(o) != 0)
        strcat(answer,strtok(o,"."));
      strcat(answer,")");
      free(o);
      return answer;
    }
  else
    return strdup(answer);
}

void
add_kelsal_list(const char * ordi, const char * salle)
{
  static struct kelsal_list * last = NULL;
  struct kelsal_list * tmp;
  
  if ( (tmp = (struct kelsal_list *) malloc(sizeof(struct kelsal_list)))
       == NULL)
    {
      fprintf( stderr , "Allocation de memoire impossible\n" );
      exit(-1);
    }

  if (regcomp(&(tmp->ordi), ordi, REG_EXTENDED))
    {
      fprintf(stderr, "regex '%s' is not valid\n", ordi);
      free(tmp);
      return;
    }

  if ((tmp->salle = strdup(salle)) == NULL)
    {
      fprintf( stderr , "Allocation de memoire impossible\n" );
      exit(-1);
    }

  if (kl == NULL)
    kl = tmp;
  else if (last != NULL)
    last->next = tmp;
  else
    {
      /* dead code */
      fprintf(stderr , "Inconsistance: ajout d'un element de kelsal_list\n");
      exit(-1);
    }

  last = tmp;
  return;
}

int
init_regex(void)
{
  if (regcomp(&silent_line, "^[ \t]*([;#].*)?$", REG_EXTENDED|REG_NOSUB))
    return 0;

  if (regcomp(&used_line,
	      "^[ \t]*\"([^\"]*)\"[ \t]+\"([^\"]*)\"[ \t]*([;#].*)?$", 
	      REG_EXTENDED))
    return 0;

  return 1;
}

void
parse_line(char * line)
{
  regmatch_t subreg[3];

  if (!regexec(&silent_line, line, 0, NULL, 0))
    {
      //printf("silent line: %s\n", line);
    }
  else if (!regexec(&used_line, line, 3, subreg, 0))
    {
      /* faites pas ca les enfants, c'est crade... */
      line[subreg[1].rm_so - 1] = '^';
      line[subreg[1].rm_eo] = '\0';
      line[subreg[2].rm_eo] = '\0';
      add_kelsal_list(line + subreg[1].rm_so - 1, line + subreg[2].rm_so);
    }
  else
    fprintf(stderr, "syntax error: %s\n", line);

  return;
}

int
read_kelsal(void)
{
  char buffer[BUFFER_SIZE];
  char line[BUFFER_SIZE + 1];
  char * chemin_kelsal;
  int buffer_index, line_index;
  size_t readed;
  FILE * fd;

  if (!init_regex())
    return 1;


  /* on prends le chemin de .pot.kelsal dans le repertoire de l utilisateur */
  if ( (chemin_kelsal = 
	(char *) malloc(sizeof(char) *
			(strlen(getenv("HOME")) + strlen(POTSAL) + 1))) 
       == NULL )
    {
      fprintf( stderr , "Allocation de memoire impossible\n" );
      exit(-1);
    }
  strcpy ( chemin_kelsal , getenv("HOME") );
  strcat ( chemin_kelsal , POTSAL );
  
  /* Si aucun des 2 fichiers ne peux etre ouverts, fini */
  if( (fd = fopen (chemin_kelsal, "r")) == NULL )
    if( (fd = fopen (POTSALDEFAULT, "r")) == NULL )
      return 2;

  line_index = 0;
  while ((readed = fread(buffer, sizeof(char), BUFFER_SIZE, fd)) > 0)
    for (buffer_index = 0; buffer_index < readed; buffer_index++)
      {
	if (line_index > BUFFER_SIZE + 1)
	  {
	    if (buffer[buffer_index] == '\n' || buffer[buffer_index] == '\r')
	      {
		line[(40>BUFFER_SIZE ? BUFFER_SIZE : 40)] = '\0';
		fprintf(stderr,
			"Parse error: Line too long (%s...)\n",
			line);
		line_index = 0;
	      }
	  }
	else
	  switch(buffer[buffer_index])
	    {
	    case '\n':
	    case '\r':
	      line[line_index] = '\0';
	      parse_line(line);
	      line_index = 0;
	      break;
		
	    default:
	      line[line_index] = buffer[buffer_index];
	      line_index++;
	      break;
	    }
      }

  free(chemin_kelsal);

  return 0;
}
